import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../index.dart';
import 'widgets.dart';

/// 麦克风检测
class MicrophoneDetection extends GetView<HardwareDetectionController> {
  MicrophoneDetection({Key? key}) : super(key: key);

  final smallTitleStyle = TextStyle(
    fontSize: 14,
    fontWeight: FontWeight.bold,
    height: 1,
  );

  /// 副标题行高：[摄像头]、[麦克风]、[扬声器]
  static const _subTitleRowHright = 30.0;

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 300,
      child: Column(
        children: [
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                width: 300,
                height: _subTitleRowHright,
                child: Row(
                  children: [
                    Text("麦克风", style: smallTitleStyle),
                    SizedBox(width: 10),
                    Obx(() {
                      if (controller.state.microphoneAvailable == null)
                        return Container();
                      return controller.state.microphoneAvailable!
                          ? CommonWidgets.deviceAvailableTip()
                          : CommonWidgets.deviceUnavailableTip();
                    }),
                    Expanded(child: Container()),
                    InkWell(
                      onTap: controller.refreshMicrophoneList,
                      child: Icon(
                        Icons.refresh,
                        size: 18,
                      ),
                    )
                  ],
                ),
              ),
              Container(width: 300, child: _microphoneSelector()),
            ],
          ),
          DetectButton(
            deviceType: HardwareDeviceType.microphone,
            subTitleRowHright: _subTitleRowHright,
          ),
        ],
      ),
    );
  }

  /// 麦克风选择器
  Widget _microphoneSelector() {
    return Obx(
      () {
        List<HardwareDevice> source = controller.state.microphoneList;
        bool noMicrophone =
            source.isEmpty || controller.state.currentMicrophoneId.isEmpty;
        source.forEach((element) {
          print(element.id);
        });
        if (controller.state.searchingMicrophone)
          return CommonWidgets.deviceFindingTip();
        return noMicrophone
            ? CommonWidgets.deviceNotFoundTip()
            : DropdownButton<String>(
                isExpanded: true,
                value: controller.state.currentMicrophoneId,
                items: source.map((HardwareDevice device) {
                  return DropdownMenuItem<String>(
                    value: device.id,
                    child: Text(device.name),
                  );
                }).toList(),
                onChanged: (String? value) {
                  if (value == null) return;
                  controller.selectMicrophoneById(value);
                },
                style: TextStyle(
                  fontSize: 14,
                ),
                dropdownColor: Colors.white,
                underline: Container(
                  height: 1,
                  color: Colors.grey,
                ),
              );
        // : FSelect<HardwareDevice, String>(
        //     source: source,
        //     height: 30,
        //     width: 300,
        //     value: controller.state.currentMicrophone?.id,
        //     optionValueExtractor: (value) {
        //       return value.id;
        //     },
        //     optionLabelExtractor: (value) {
        //       return value.name;
        //     },
        //     onSelectChanged: (value, index) {
        //       if (value == null) return;
        //       controller.selectMicrophoneById(value);
        //     },
        //     fontSize: 14,
        //   );
      },
    );
  }
}
